# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from airflow.providers.edge3.models.db import EdgeDBManager


class TestMigrationEnv:
    """Test edge3 migration env configuration."""

    def test_version_table_name(self):
        assert EdgeDBManager.version_table_name == "alembic_version_edge3"

    def test_target_metadata_contains_edge_tables(self):
        table_names = set(EdgeDBManager.metadata.tables.keys())
        assert len(table_names) == 3
        assert "edge_worker" in table_names
        assert "edge_job" in table_names
        assert "edge_logs" in table_names
